@@ -2,15 +2,14 @@
 # Copyright (c) 2019 Ansible Project
 # Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)
 
-from __future__ import absolute_import, division, print_function
-__metaclass__ = type
+from __future__ import annotations
 
+import json
 import os
 import re
 
 from ast import literal_eval
-from ansible.module_utils._text import to_native
-from ansible.module_utils.common._json_compat import json
+from ansible.module_utils.common.text.converters import to_native
 from ansible.module_utils.common.collections import is_iterable
 from ansible.module_utils.common.text.converters import jsonify
 from ansible.module_utils.common.text.formatters import human_to_bytes
@@ -75,6 +74,8 @@ def check_mutually_exclusive(terms, parameters, options_context=None):
 
     :arg terms: List of mutually exclusive parameters
     :arg parameters: Dictionary of parameters
+    :kwarg options_context: List of strings of parent key names if ``terms`` are
+        in a sub spec.
 
     :returns: Empty list or raises :class:`TypeError` if the check fails.
     """
@@ -142,6 +143,8 @@ def check_required_together(terms, parameters, options_context=None):
         parameters that are all required when at least one is specified
         in the parameters.
     :arg parameters: Dictionary of parameters
+    :kwarg options_context: List of strings of parent key names if ``terms`` are
+        in a sub spec.
 
     :returns: Empty list or raises :class:`TypeError` if the check fails.
     """
@@ -174,6 +177,8 @@ def check_required_by(requirements, parameters, options_context=None):
 
     :arg requirements: Dictionary of requirements
     :arg parameters: Dictionary of parameters
+    :kwarg options_context: List of strings of parent key names if ``requirements`` are
+        in a sub spec.
 
     :returns: Empty dictionary or raises :class:`TypeError` if the
     """
@@ -213,6 +218,8 @@ def check_required_arguments(argument_spec, parameters, options_context=None):
     :arg argument_spec: Argument spec dictionary containing all parameters
         and their specification
     :arg parameters: Dictionary of parameters
+    :kwarg options_context: List of strings of parent key names if ``argument_spec`` are
+        in a sub spec.
 
     :returns: Empty list or raises :class:`TypeError` if the check fails.
     """
@@ -280,6 +287,8 @@ def check_required_if(requirements, parameters, options_context=None):
                 }
             ]
 
+    :kwarg options_context: List of strings of parent key names if ``requirements`` are
+        in a sub spec.
     """
     results = []
     if requirements is None:
@@ -371,7 +380,7 @@ def check_type_str(value, allow_conversion=True, param=None, prefix=''):
     if isinstance(value, string_types):
         return value
 
-    if allow_conversion:
+    if allow_conversion and value is not None:
         return to_native(value, errors='surrogate_or_strict')
 
     msg = "'{0!r}' is not a string and conversion is not allowed".format(value)
@@ -533,7 +542,7 @@ def check_type_raw(value):
 def check_type_bytes(value):
     """Convert a human-readable string value to bytes
 
-    Raises :class:`TypeError` if unable to covert the value
+    Raises :class:`TypeError` if unable to convert the value
     """
     try:
         return human_to_bytes(value)
@@ -546,7 +555,7 @@ def check_type_bits(value):
 
     Example: ``check_type_bits('1Mb')`` returns integer 1048576.
 
-    Raises :class:`TypeError` if unable to covert the value.
+    Raises :class:`TypeError` if unable to convert the value.
     """
     try:
         return human_to_bytes(value, isbits=True)
@@ -558,7 +567,7 @@ def check_type_jsonarg(value):
     """Return a jsonified string. Sometimes the controller turns a json string
     into a dict/list so transform it back into json here
 
-    Raises :class:`TypeError` if unable to covert the value
+    Raises :class:`TypeError` if unable to convert the value
 
     """
     if isinstance(value, (text_type, binary_type)):
