@@ -1,11 +1,9 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 
 # Copyright: (c) 2012, Jan-Piet Mens <jpmens () gmail.com>
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-from __future__ import absolute_import, division, print_function
-__metaclass__ = type
+from __future__ import annotations
 
 
 DOCUMENTATION = r'''
@@ -23,10 +21,26 @@ description:
      - HTTP redirects can redirect from HTTP to HTTPS so you should be sure that
        your proxy environment for both protocols is correct.
      - From Ansible 2.4 when run with C(--check), it will do a HEAD request to validate the URL but
-       will not download the entire file or verify it against hashes.
+       will not download the entire file or verify it against hashes and will report incorrect changed status.
      - For Windows targets, use the M(ansible.windows.win_get_url) module instead.
 version_added: '0.6'
 options:
+  ciphers:
+    description:
+      - SSL/TLS Ciphers to use for the request
+      - 'When a list is provided, all ciphers are joined in order with V(:)'
+      - See the L(OpenSSL Cipher List Format,https://www.openssl.org/docs/manmaster/man1/openssl-ciphers.html#CIPHER-LIST-FORMAT)
+        for more details.
+      - The available ciphers is dependent on the Python and OpenSSL/LibreSSL versions
+    type: list
+    elements: str
+    version_added: '2.14'
+  decompress:
+    description:
+      - Whether to attempt to decompress gzip content-encoded responses
+    type: bool
+    default: true
+    version_added: '2.14'
   url:
     description:
       - HTTP, HTTPS, or FTP URL in the form (http|https|ftp)://[user[:pass]]@host.domain[:port]/path
@@ -35,32 +49,31 @@ options:
   dest:
     description:
       - Absolute path of where to download the file to.
-      - If C(dest) is a directory, either the server provided filename or, if
+      - If O(dest) is a directory, either the server provided filename or, if
         none provided, the base name of the URL on the remote server will be
-        used. If a directory, C(force) has no effect.
-      - If C(dest) is a directory, the file will always be downloaded
-        (regardless of the C(force) option), but replaced only if the contents changed..
+        used. If a directory, O(force) has no effect.
+      - If O(dest) is a directory, the file will always be downloaded
+        (regardless of the O(force) and O(checksum) option), but
+        replaced only if the contents changed.
     type: path
     required: true
   tmp_dest:
     description:
       - Absolute path of where temporary file is downloaded to.
       - When run on Ansible 2.5 or greater, path defaults to ansible's remote_tmp setting
-      - When run on Ansible prior to 2.5, it defaults to C(TMPDIR), C(TEMP) or C(TMP) env variables or a platform specific value.
-      - U(https://docs.python.org/2/library/tempfile.html#tempfile.tempdir)
+      - When run on Ansible prior to 2.5, it defaults to E(TMPDIR), E(TEMP) or E(TMP) env variables or a platform specific value.
+      - U(https://docs.python.org/3/library/tempfile.html#tempfile.tempdir)
     type: path
     version_added: '2.1'
   force:
     description:
-      - If C(yes) and C(dest) is not a directory, will download the file every
-        time and replace the file if the contents change. If C(no), the file
+      - If V(true) and O(dest) is not a directory, will download the file every
+        time and replace the file if the contents change. If V(false), the file
         will only be downloaded if the destination does not exist. Generally
-        should be C(yes) only for small local files.
-      - Prior to 0.6, this module behaved as if C(yes) was the default.
-      - Alias C(thirsty) has been deprecated and will be removed in 2.13.
+        should be V(true) only for small local files.
+      - Prior to 0.6, this module behaved as if V(true) was the default.
     type: bool
     default: no
-    aliases: [ thirsty ]
     version_added: '0.7'
   backup:
     description:
@@ -69,16 +82,6 @@ options:
     type: bool
     default: no
     version_added: '2.1'
-  sha256sum:
-    description:
-      - If a SHA-256 checksum is passed to this parameter, the digest of the
-        destination file will be calculated after it is downloaded to ensure
-        its integrity and verify that the transfer completed successfully.
-        This option is deprecated and will be removed in version 2.14. Use
-        option C(checksum) instead.
-    default: ''
-    type: str
-    version_added: "1.3"
   checksum:
     description:
       - 'If a checksum is passed to this parameter, the digest of the
@@ -88,24 +91,26 @@ options:
         checksum="sha256:http://example.com/path/sha256sum.txt"'
       - If you worry about portability, only the sha1 algorithm is available
         on all platforms and python versions.
-      - The third party hashlib library can be installed for access to additional algorithms.
+      - The Python ``hashlib`` module is responsible for providing the available algorithms.
+        The choices vary based on Python version and OpenSSL version.
+      - On systems running in FIPS compliant mode, the ``md5`` algorithm may be unavailable.
       - Additionally, if a checksum is passed to this parameter, and the file exist under
-        the C(dest) location, the I(destination_checksum) would be calculated, and if
-        checksum equals I(destination_checksum), the file download would be skipped
-        (unless C(force) is true). If the checksum does not equal I(destination_checksum),
+        the O(dest) location, the C(destination_checksum) would be calculated, and if
+        checksum equals C(destination_checksum), the file download would be skipped
+        (unless O(force) is V(true)). If the checksum does not equal C(destination_checksum),
         the destination file is deleted.
     type: str
     default: ''
     version_added: "2.0"
   use_proxy:
     description:
-      - if C(no), it will not use a proxy, even if one is defined in
+      - if V(false), it will not use a proxy, even if one is defined in
         an environment variable on the target hosts.
     type: bool
     default: yes
   validate_certs:
     description:
-      - If C(no), SSL certificates will not be validated.
+      - If V(false), SSL certificates will not be validated.
       - This should only be used on personally controlled sites using self-signed certificates.
     type: bool
     default: yes
@@ -126,16 +131,16 @@ options:
   url_username:
     description:
       - The username for use in HTTP basic authentication.
-      - This parameter can be used without C(url_password) for sites that allow empty passwords.
-      - Since version 2.8 you can also use the C(username) alias for this option.
+      - This parameter can be used without O(url_password) for sites that allow empty passwords.
+      - Since version 2.8 you can also use the O(username) alias for this option.
     type: str
     aliases: ['username']
     version_added: '1.6'
   url_password:
     description:
         - The password for use in HTTP basic authentication.
-        - If the C(url_username) parameter is not specified, the C(url_password) parameter will not be used.
-        - Since version 2.8 you can also use the 'password' alias for this option.
+        - If the O(url_username) parameter is not specified, the O(url_password) parameter will not be used.
+        - Since version 2.8 you can also use the O(password) alias for this option.
     type: str
     aliases: ['password']
     version_added: '1.6'
@@ -151,13 +156,13 @@ options:
   client_cert:
     description:
       - PEM formatted certificate chain file to be used for SSL client authentication.
-      - This file can also include the key as well, and if the key is included, C(client_key) is not required.
+      - This file can also include the key as well, and if the key is included, O(client_key) is not required.
     type: path
     version_added: '2.4'
   client_key:
     description:
       - PEM formatted file that contains your private key to be used for SSL client authentication.
-      - If C(client_cert) contains both the certificate and key, this option is not required.
+      - If O(client_cert) contains both the certificate and key, this option is not required.
     type: path
     version_added: '2.4'
   http_agent:
@@ -165,20 +170,46 @@ options:
       - Header to identify as, generally appears in web server logs.
     type: str
     default: ansible-httpget
+  unredirected_headers:
+    description:
+      - A list of header names that will not be sent on subsequent redirected requests. This list is case
+        insensitive. By default all headers will be redirected. In some cases it may be beneficial to list
+        headers such as C(Authorization) here to avoid potential credential exposure.
+    default: []
+    type: list
+    elements: str
+    version_added: '2.12'
   use_gssapi:
     description:
       - Use GSSAPI to perform the authentication, typically this is for Kerberos or Kerberos through Negotiate
         authentication.
       - Requires the Python library L(gssapi,https://github.com/pythongssapi/python-gssapi) to be installed.
-      - Credentials for GSSAPI can be specified with I(url_username)/I(url_password) or with the GSSAPI env var
+      - Credentials for GSSAPI can be specified with O(url_username)/O(url_password) or with the GSSAPI env var
         C(KRB5CCNAME) that specified a custom Kerberos credential cache.
-      - NTLM authentication is C(not) supported even if the GSSAPI mech for NTLM has been installed.
+      - NTLM authentication is I(not) supported even if the GSSAPI mech for NTLM has been installed.
     type: bool
     default: no
     version_added: '2.11'
+  use_netrc:
+    description:
+      - Determining whether to use credentials from ``~/.netrc`` file
+      - By default .netrc is used with Basic authentication headers
+      - When set to False, .netrc credentials are ignored
+    type: bool
+    default: true
+    version_added: '2.14'
 # informational: requirements for nodes
 extends_documentation_fragment:
     - files
+    - action_common_attributes
+attributes:
+    check_mode:
+        details: the changed status will reflect comparison to an empty source file
+        support: partial
+    diff_mode:
+        support: none
+    platform:
+        platforms: posix
 notes:
      - For Windows targets, use the M(ansible.windows.win_get_url) module instead.
 seealso:
@@ -190,19 +221,19 @@ author:
 
 EXAMPLES = r'''
 - name: Download foo.conf
-  get_url:
+  ansible.builtin.get_url:
     url: http://example.com/path/file.conf
     dest: /etc/foo.conf
     mode: '0440'
 
 - name: Download file and force basic auth
-  get_url:
+  ansible.builtin.get_url:
     url: http://example.com/path/file.conf
     dest: /etc/foo.conf
     force_basic_auth: yes
 
 - name: Download file with custom HTTP headers
-  get_url:
+  ansible.builtin.get_url:
     url: http://example.com/path/file.conf
     dest: /etc/foo.conf
     headers:
@@ -210,31 +241,31 @@ EXAMPLES = r'''
       key2: two
 
 - name: Download file with check (sha256)
-  get_url:
+  ansible.builtin.get_url:
     url: http://example.com/path/file.conf
     dest: /etc/foo.conf
     checksum: sha256:b5bb9d8014a0f9b1d61e21e796d78dccdf1352f23cd32812f4850b878ae4944c
 
 - name: Download file with check (md5)
-  get_url:
+  ansible.builtin.get_url:
     url: http://example.com/path/file.conf
     dest: /etc/foo.conf
     checksum: md5:66dffb5228a211e61d6d7ef4a86f5758
 
 - name: Download file with checksum url (sha256)
-  get_url:
+  ansible.builtin.get_url:
     url: http://example.com/path/file.conf
     dest: /etc/foo.conf
     checksum: sha256:http://example.com/path/sha256sum.txt
 
 - name: Download file from a file path
-  get_url:
-    url: file:///tmp/afile.txt
+  ansible.builtin.get_url:
+    url: file:///tmp/a_file.txt
     dest: /tmp/afilecopy.txt
 
 - name: < Fetch file that requires authentication.
         username/password only available since 2.8, in older versions you need to use url_username/url_password
-  get_url:
+  ansible.builtin.get_url:
     url: http://example.com/path/file.conf
     dest: /etc/foo.conf
     username: bar
@@ -334,7 +365,6 @@ url:
     sample: https://www.ansible.com/
 '''
 
-import datetime
 import os
 import re
 import shutil
@@ -343,7 +373,8 @@ import traceback
 
 from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils.six.moves.urllib.parse import urlsplit
-from ansible.module_utils._text import to_native
+from ansible.module_utils.compat.datetime import utcnow, utcfromtimestamp
+from ansible.module_utils.common.text.converters import to_native
 from ansible.module_utils.urls import fetch_url, url_argument_spec
 
 # ==============================================================
@@ -357,16 +388,18 @@ def url_filename(url):
     return fn
 
 
-def url_get(module, url, dest, use_proxy, last_mod_time, force, timeout=10, headers=None, tmp_dest='', method='GET'):
+def url_get(module, url, dest, use_proxy, last_mod_time, force, timeout=10, headers=None, tmp_dest='', method='GET', unredirected_headers=None,
+            decompress=True, ciphers=None, use_netrc=True):
     """
     Download data from the url and store in a temporary file.
 
     Return (tempfile, info about the request)
     """
 
-    start = datetime.datetime.utcnow()
-    rsp, info = fetch_url(module, url, use_proxy=use_proxy, force=force, last_mod_time=last_mod_time, timeout=timeout, headers=headers, method=method)
-    elapsed = (datetime.datetime.utcnow() - start).seconds
+    start = utcnow()
+    rsp, info = fetch_url(module, url, use_proxy=use_proxy, force=force, last_mod_time=last_mod_time, timeout=timeout, headers=headers, method=method,
+                          unredirected_headers=unredirected_headers, decompress=decompress, ciphers=ciphers, use_netrc=use_netrc)
+    elapsed = (utcnow() - start).seconds
 
     if info['status'] == 304:
         module.exit_json(url=url, dest=dest, changed=False, msg=info.get('msg', ''), status_code=info['status'], elapsed=elapsed)
@@ -445,11 +478,14 @@ def main():
         url=dict(type='str', required=True),
         dest=dict(type='path', required=True),
         backup=dict(type='bool', default=False),
-        sha256sum=dict(type='str', default=''),
         checksum=dict(type='str', default=''),
         timeout=dict(type='int', default=10),
         headers=dict(type='dict'),
         tmp_dest=dict(type='path'),
+        unredirected_headers=dict(type='list', elements='str', default=[]),
+        decompress=dict(type='bool', default=True),
+        ciphers=dict(type='list', elements='str'),
+        use_netrc=dict(type='bool', default=True),
     )
 
     module = AnsibleModule(
@@ -457,27 +493,21 @@ def main():
         argument_spec=argument_spec,
         add_file_common_args=True,
         supports_check_mode=True,
-        mutually_exclusive=[['checksum', 'sha256sum']],
     )
 
-    if module.params.get('thirsty'):
-        module.deprecate('The alias "thirsty" has been deprecated and will be removed, use "force" instead',
-                         version='2.13', collection_name='ansible.builtin')
-
-    if module.params.get('sha256sum'):
-        module.deprecate('The parameter "sha256sum" has been deprecated and will be removed, use "checksum" instead',
-                         version='2.14', collection_name='ansible.builtin')
-
     url = module.params['url']
     dest = module.params['dest']
     backup = module.params['backup']
     force = module.params['force']
-    sha256sum = module.params['sha256sum']
     checksum = module.params['checksum']
     use_proxy = module.params['use_proxy']
     timeout = module.params['timeout']
     headers = module.params['headers']
     tmp_dest = module.params['tmp_dest']
+    unredirected_headers = module.params['unredirected_headers']
+    decompress = module.params['decompress']
+    ciphers = module.params['ciphers']
+    use_netrc = module.params['use_netrc']
 
     result = dict(
         changed=False,
@@ -491,10 +521,6 @@ def main():
     dest_is_dir = os.path.isdir(dest)
     last_mod_time = None
 
-    # workaround for usage of deprecated sha256sum parameter
-    if sha256sum:
-        checksum = 'sha256:%s' % (sha256sum)
-
     # checksum specified, parse for algorithm and checksum
     if checksum:
         try:
@@ -505,23 +531,30 @@ def main():
         if is_url(checksum):
             checksum_url = checksum
             # download checksum file to checksum_tmpsrc
-            checksum_tmpsrc, checksum_info = url_get(module, checksum_url, dest, use_proxy, last_mod_time, force, timeout, headers, tmp_dest)
+            checksum_tmpsrc, checksum_info = url_get(module, checksum_url, dest, use_proxy, last_mod_time, force, timeout, headers, tmp_dest,
+                                                     unredirected_headers=unredirected_headers, ciphers=ciphers, use_netrc=use_netrc)
             with open(checksum_tmpsrc) as f:
                 lines = [line.rstrip('\n') for line in f]
             os.remove(checksum_tmpsrc)
             checksum_map = []
-            for line in lines:
-                # Split by one whitespace to keep the leading type char ' ' (whitespace) for text and '*' for binary
-                parts = line.split(" ", 1)
-                if len(parts) == 2:
-                    # Remove the leading type char, we expect
-                    if parts[1].startswith((" ", "*",)):
-                        parts[1] = parts[1][1:]
-
-                    # Append checksum and path without potential leading './'
-                    checksum_map.append((parts[0], parts[1].lstrip("./")))
-
             filename = url_filename(url)
+            if len(lines) == 1 and len(lines[0].split()) == 1:
+                # Only a single line with a single string
+                # treat it as a checksum only file
+                checksum_map.append((lines[0], filename))
+            else:
+                # The assumption here is the file is in the format of
+                # checksum filename
+                for line in lines:
+                    # Split by one whitespace to keep the leading type char ' ' (whitespace) for text and '*' for binary
+                    parts = line.split(" ", 1)
+                    if len(parts) == 2:
+                        # Remove the leading type char, we expect
+                        if parts[1].startswith((" ", "*",)):
+                            parts[1] = parts[1][1:]
+
+                        # Append checksum and path without potential leading './'
+                        checksum_map.append((parts[0], parts[1].lstrip("./")))
 
             # Look through each line in the checksum file for a hash corresponding to
             # the filename in the url, returning the first hash that is found.
@@ -566,7 +599,7 @@ def main():
         # If the file already exists, prepare the last modified time for the
         # request.
         mtime = os.path.getmtime(dest)
-        last_mod_time = datetime.datetime.utcfromtimestamp(mtime)
+        last_mod_time = utcfromtimestamp(mtime)
 
         # If the checksum does not match we have to force the download
         # because last_mod_time may be newer than on remote
@@ -574,10 +607,11 @@ def main():
             force = True
 
     # download to tmpsrc
-    start = datetime.datetime.utcnow()
+    start = utcnow()
     method = 'HEAD' if module.check_mode else 'GET'
-    tmpsrc, info = url_get(module, url, dest, use_proxy, last_mod_time, force, timeout, headers, tmp_dest, method)
-    result['elapsed'] = (datetime.datetime.utcnow() - start).seconds
+    tmpsrc, info = url_get(module, url, dest, use_proxy, last_mod_time, force, timeout, headers, tmp_dest, method,
+                           unredirected_headers=unredirected_headers, decompress=decompress, ciphers=ciphers, use_netrc=use_netrc)
+    result['elapsed'] = (utcnow() - start).seconds
     result['src'] = tmpsrc
 
     # Now the request has completed, we can finally generate the final
