@@ -3,8 +3,7 @@
 # (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-from __future__ import absolute_import, division, print_function
-__metaclass__ = type
+from __future__ import annotations
 
 
 DOCUMENTATION = '''
@@ -29,15 +28,15 @@ options:
         description:
             - The path of where the repository should be checked out. This
               is equivalent to C(git clone [repo_url] [directory]). The repository
-              named in I(repo) is not appended to this path and the destination directory must be empty. This
-              parameter is required, unless I(clone) is set to C(no).
+              named in O(repo) is not appended to this path and the destination directory must be empty. This
+              parameter is required, unless O(clone) is set to V(false).
         type: path
         required: true
     version:
         description:
             - What version of the repository to check out. This can be
-              the literal string C(HEAD), a branch name, a tag name.
-              It can also be a I(SHA-1) hash, in which case I(refspec) needs
+              the literal string V(HEAD), a branch name, a tag name.
+              It can also be a I(SHA-1) hash, in which case O(refspec) needs
               to be specified if the given revision is not already available.
         type: str
         default: "HEAD"
@@ -45,7 +44,7 @@ options:
         description:
             - Will ensure or not that "-o StrictHostKeyChecking=no" is present as an ssh option.
             - Be aware that this disables a protection against MITM attacks.
-            - Those using OpenSSH >= 7.5 might want to set I(ssh_opt) to 'StrictHostKeyChecking=accept-new'
+            - Those using OpenSSH >= 7.5 might want to set O(ssh_opts) to V(StrictHostKeyChecking=accept-new)
               instead, it does not remove the MITM issue but it does restrict it to the first attempt.
         type: bool
         default: 'no'
@@ -54,7 +53,7 @@ options:
         description:
             - As of OpenSSH 7.5, "-o StrictHostKeyChecking=accept-new" can be
               used which is safer and will only accepts host keys which are
-              not present or are the same. if C(yes), ensure that
+              not present or are the same. if V(true), ensure that
               "-o StrictHostKeyChecking=accept-new" is present as an ssh option.
         type: bool
         default: 'no'
@@ -62,12 +61,12 @@ options:
     ssh_opts:
         description:
             - Options git will pass to ssh when used as protocol, it works via C(git)'s
-              GIT_SSH/GIT_SSH_COMMAND environment variables.
-            - For older versions it appends GIT_SSH_OPTS (specific to this module) to the
+              E(GIT_SSH)/E(GIT_SSH_COMMAND) environment variables.
+            - For older versions it appends E(GIT_SSH_OPTS) (specific to this module) to the
               variables above or via a wrapper script.
-            - Other options can add to this list, like I(key_file) and I(accept_hostkey).
+            - Other options can add to this list, like O(key_file) and O(accept_hostkey).
             - An example value could be "-o StrictHostKeyChecking=no" (although this particular
-              option is better set by I(accept_hostkey)).
+              option is better set by O(accept_hostkey)).
             - The module ensures that 'BatchMode=yes' is always present to avoid prompts.
         type: str
         version_added: "1.5"
@@ -75,12 +74,13 @@ options:
     key_file:
         description:
             - Specify an optional private key file path, on the target host, to use for the checkout.
-            - This ensures 'IdentitiesOnly=yes' is present in ssh_opts.
+            - This ensures 'IdentitiesOnly=yes' is present in O(ssh_opts).
         type: path
         version_added: "1.5"
     reference:
         description:
             - Reference repository (see "git clone --reference ...").
+        type: str
         version_added: "1.4"
     remote:
         description:
@@ -99,29 +99,29 @@ options:
         version_added: "1.9"
     force:
         description:
-            - If C(yes), any modified files in the working
+            - If V(true), any modified files in the working
               repository will be discarded.  Prior to 0.7, this was always
-              C(yes) and could not be disabled.  Prior to 1.9, the default was
-              C(yes).
+              V(true) and could not be disabled.  Prior to 1.9, the default was
+              V(true).
         type: bool
         default: 'no'
         version_added: "0.7"
     depth:
         description:
             - Create a shallow clone with a history truncated to the specified
-              number or revisions. The minimum possible value is C(1), otherwise
+              number or revisions. The minimum possible value is V(1), otherwise
               ignored. Needs I(git>=1.9.1) to work correctly.
         type: int
         version_added: "1.2"
     clone:
         description:
-            - If C(no), do not clone the repository even if it does not exist locally.
+            - If V(false), do not clone the repository even if it does not exist locally.
         type: bool
         default: 'yes'
         version_added: "1.9"
     update:
         description:
-            - If C(no), do not retrieve new revisions from the origin repository.
+            - If V(false), do not retrieve new revisions from the origin repository.
             - Operations like archive will work on the existing (old) repository and might
               not respond to changes to the options version or remote.
         type: bool
@@ -135,7 +135,7 @@ options:
         version_added: "1.4"
     bare:
         description:
-            - If C(yes), repository will be created as a bare repo, otherwise
+            - If V(true), repository will be created as a bare repo, otherwise
               it will be a standard repo with a workspace.
         type: bool
         default: 'no'
@@ -149,7 +149,7 @@ options:
 
     recursive:
         description:
-            - If C(no), repository will be cloned without the --recursive
+            - If V(false), repository will be cloned without the C(--recursive)
               option, skipping sub-modules.
         type: bool
         default: 'yes'
@@ -164,10 +164,10 @@ options:
 
     track_submodules:
         description:
-            - If C(yes), submodules will track the latest commit on their
+            - If V(true), submodules will track the latest commit on their
               master branch (or other branch specified in .gitmodules).  If
-              C(no), submodules will be kept at the revision specified by the
-              main project. This is equivalent to specifying the --remote flag
+              V(false), submodules will be kept at the revision specified by the
+              main project. This is equivalent to specifying the C(--remote) flag
               to git submodule update.
         type: bool
         default: 'no'
@@ -175,7 +175,7 @@ options:
 
     verify_commit:
         description:
-            - If C(yes), when cloning or checking out a I(version) verify the
+            - If V(true), when cloning or checking out a O(version) verify the
               signature of a GPG signed commit. This requires git version>=2.1.0
               to be installed. The commit MUST be signed and the public key MUST
               be present in the GPG keyring.
@@ -196,7 +196,7 @@ options:
 
     archive_prefix:
         description:
-            - Specify a prefix to add to each file path in archive. Requires I(archive) to be specified.
+            - Specify a prefix to add to each file path in archive. Requires O(archive) to be specified.
         version_added: "2.10"
         type: str
 
@@ -207,15 +207,18 @@ options:
         type: path
         version_added: "2.7"
 
-    gpg_whitelist:
+    gpg_allowlist:
         description:
            - A list of trusted GPG fingerprints to compare to the fingerprint of the
              GPG-signed commit.
-           - Only used when I(verify_commit=yes).
+           - Only used when O(verify_commit=yes).
            - Use of this feature requires Git 2.6+ due to its reliance on git's C(--raw) flag to C(verify-commit) and C(verify-tag).
+           - Alias O(gpg_allowlist) is added in version 2.17.
+           - Alias O(gpg_whitelist) is deprecated and will be removed in version 2.21.
         type: list
         elements: str
         default: []
+        aliases: [ gpg_whitelist ]
         version_added: "2.9"
 
 requirements:
@@ -337,7 +340,7 @@ import shutil
 import tempfile
 from ansible.module_utils.compat.version import LooseVersion
 
-from ansible.module_utils._text import to_native, to_text
+from ansible.module_utils.common.text.converters import to_native, to_text
 from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils.common.locale import get_best_parsable_locale
 from ansible.module_utils.common.process import get_bin_path
@@ -567,7 +570,7 @@ def get_submodule_versions(git_path, module, dest, version='HEAD'):
 
 
 def clone(git_path, module, repo, dest, remote, depth, version, bare,
-          reference, refspec, git_version_used, verify_commit, separate_git_dir, result, gpg_whitelist, single_branch):
+          reference, refspec, git_version_used, verify_commit, separate_git_dir, result, gpg_allowlist, single_branch):
     ''' makes a new git repo if it does not already exist '''
     dest_dirname = os.path.dirname(dest)
     try:
@@ -634,7 +637,7 @@ def clone(git_path, module, repo, dest, remote, depth, version, bare,
         module.run_command(cmd, check_rc=True, cwd=dest)
 
     if verify_commit:
-        verify_commit_sign(git_path, module, dest, version, gpg_whitelist)
+        verify_commit_sign(git_path, module, dest, version, gpg_allowlist)
 
 
 def has_local_mods(module, git_path, dest, bare):
@@ -825,7 +828,7 @@ def get_head_branch(git_path, module, dest, remote, bare=False):
         repo_path = get_repo_path(dest, bare)
     except (IOError, ValueError) as err:
         # No repo path found
-        """``.git`` file does not have a valid format for detached Git dir."""
+        # ``.git`` file does not have a valid format for detached Git dir.
         module.fail_json(
             msg='Current repo does not have a valid reference to a '
             'separate Git dir or it refers to the invalid path',
@@ -1015,7 +1018,7 @@ def set_remote_branch(git_path, module, dest, remote, version, depth):
         module.fail_json(msg="Failed to fetch branch from remote: %s" % version, stdout=out, stderr=err, rc=rc)
 
 
-def switch_version(git_path, module, dest, remote, version, verify_commit, depth, gpg_whitelist):
+def switch_version(git_path, module, dest, remote, version, verify_commit, depth, gpg_allowlist):
     cmd = ''
     if version == 'HEAD':
         branch = get_head_branch(git_path, module, dest, remote)
@@ -1051,26 +1054,26 @@ def switch_version(git_path, module, dest, remote, version, verify_commit, depth
                              stdout=out1, stderr=err1, rc=rc, cmd=cmd)
 
     if verify_commit:
-        verify_commit_sign(git_path, module, dest, version, gpg_whitelist)
+        verify_commit_sign(git_path, module, dest, version, gpg_allowlist)
 
     return (rc, out1, err1)
 
 
-def verify_commit_sign(git_path, module, dest, version, gpg_whitelist):
+def verify_commit_sign(git_path, module, dest, version, gpg_allowlist):
     if version in get_annotated_tags(git_path, module, dest):
         git_sub = "verify-tag"
     else:
         git_sub = "verify-commit"
     cmd = "%s %s %s" % (git_path, git_sub, version)
-    if gpg_whitelist:
+    if gpg_allowlist:
         cmd += " --raw"
     (rc, out, err) = module.run_command(cmd, cwd=dest)
     if rc != 0:
         module.fail_json(msg='Failed to verify GPG signature of commit/tag "%s"' % version, stdout=out, stderr=err, rc=rc)
-    if gpg_whitelist:
+    if gpg_allowlist:
         fingerprint = get_gpg_fingerprint(err)
-        if fingerprint not in gpg_whitelist:
-            module.fail_json(msg='The gpg_whitelist does not include the public key "%s" for this commit' % fingerprint, stdout=out, stderr=err, rc=rc)
+        if fingerprint not in gpg_allowlist:
+            module.fail_json(msg='The gpg_allowlist does not include the public key "%s" for this commit' % fingerprint, stdout=out, stderr=err, rc=rc)
     return (rc, out, err)
 
 
@@ -1123,7 +1126,7 @@ def create_archive(git_path, module, dest, archive, archive_prefix, version, rep
     """ Helper function for creating archive using git_archive """
     all_archive_fmt = {'.zip': 'zip', '.gz': 'tar.gz', '.tar': 'tar',
                        '.tgz': 'tgz'}
-    _, archive_ext = os.path.splitext(archive)
+    dummy, archive_ext = os.path.splitext(archive)
     archive_fmt = all_archive_fmt.get(archive_ext, None)
     if archive_fmt is None:
         module.fail_json(msg="Unable to get file extension from "
@@ -1183,7 +1186,16 @@ def main():
             clone=dict(default='yes', type='bool'),
             update=dict(default='yes', type='bool'),
             verify_commit=dict(default='no', type='bool'),
-            gpg_whitelist=dict(default=[], type='list', elements='str'),
+            gpg_allowlist=dict(
+                default=[], type='list', aliases=['gpg_whitelist'], elements='str',
+                deprecated_aliases=[
+                    dict(
+                        name='gpg_whitelist',
+                        version='2.21',
+                        collection_name='ansible.builtin',
+                    )
+                ],
+            ),
             accept_hostkey=dict(default='no', type='bool'),
             accept_newhostkey=dict(default='no', type='bool'),
             key_file=dict(default=None, type='path', required=False),
@@ -1214,7 +1226,7 @@ def main():
     allow_clone = module.params['clone']
     bare = module.params['bare']
     verify_commit = module.params['verify_commit']
-    gpg_whitelist = module.params['gpg_whitelist']
+    gpg_allowlist = module.params['gpg_allowlist']
     reference = module.params['reference']
     single_branch = module.params['single_branch']
     git_path = module.params['executable'] or module.get_bin_path('git', True)
@@ -1263,7 +1275,7 @@ def main():
     # We screenscrape a huge amount of git commands so use C locale anytime we
     # call run_command()
     locale = get_best_parsable_locale(module)
-    module.run_command_environ_update = dict(LANG=locale, LC_ALL=locale, LC_MESSAGES=locale, LC_CTYPE=locale)
+    module.run_command_environ_update = dict(LANG=locale, LC_ALL=locale, LC_MESSAGES=locale, LC_CTYPE=locale, LANGUAGE=locale)
 
     if separate_git_dir:
         separate_git_dir = os.path.realpath(separate_git_dir)
@@ -1282,7 +1294,7 @@ def main():
                     repo_path = separate_git_dir
         except (IOError, ValueError) as err:
             # No repo path found
-            """``.git`` file does not have a valid format for detached Git dir."""
+            # ``.git`` file does not have a valid format for detached Git dir.
             module.fail_json(
                 msg='Current repo does not have a valid reference to a '
                 'separate Git dir or it refers to the invalid path',
@@ -1321,7 +1333,7 @@ def main():
             module.exit_json(**result)
         # there's no git config, so clone
         clone(git_path, module, repo, dest, remote, depth, version, bare, reference,
-              refspec, git_version_used, verify_commit, separate_git_dir, result, gpg_whitelist, single_branch)
+              refspec, git_version_used, verify_commit, separate_git_dir, result, gpg_allowlist, single_branch)
     elif not update:
         # Just return having found a repo already in the dest path
         # this does no checking that the repo is the actual repo
@@ -1376,7 +1388,7 @@ def main():
     # switch to version specified regardless of whether
     # we got new revisions from the repository
     if not bare:
-        switch_version(git_path, module, dest, remote, version, verify_commit, depth, gpg_whitelist)
+        switch_version(git_path, module, dest, remote, version, verify_commit, depth, gpg_allowlist)
 
     # Deal with submodules
     submodules_updated = False
