@@ -2,8 +2,7 @@
 
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-from __future__ import absolute_import, division, print_function
-__metaclass__ = type
+from __future__ import annotations
 
 
 DOCUMENTATION = r'''
@@ -40,7 +39,7 @@ options:
     version_added: "2.2"
   depth:
     description:
-      - When using C(dir), this module will, by default, recursively go through each sub directory and load up the
+      - When using O(dir), this module will, by default, recursively go through each sub directory and load up the
         variables. By explicitly setting the depth, this module will only go as deep as the depth.
     type: int
     default: 0
@@ -58,7 +57,7 @@ options:
     version_added: "2.2"
   extensions:
     description:
-      - List of file extensions to read when using C(dir).
+      - List of file extensions to read when using O(dir).
     type: list
     elements: str
     default: [ json, yaml, yml ]
@@ -73,8 +72,9 @@ options:
     version_added: "2.7"
   hash_behaviour:
     description:
-      - If set to C(merge), merges existing hash variables instead of overwriting them.
-      - If omitted C(null), the behavior falls back to the global I(hash_behaviour) configuration.
+      - If set to V(merge), merges existing hash variables instead of overwriting them.
+      - If omitted (V(null)), the behavior falls back to the global C(hash_behaviour) configuration.
+      - This option is self-contained and does not apply to individual files in O(dir). You can use a loop to apply O(hash_behaviour) per file.
     default: null
     type: str
     choices: ["replace", "merge"]
@@ -90,7 +90,7 @@ extends_documentation_fragment:
     - action_core
 attributes:
     action:
-        details: While the action plugin does do some of the work it relies on the core engine to actually create the variables, that part cannot be overriden
+        details: While the action plugin does do some of the work it relies on the core engine to actually create the variables, that part cannot be overridden
         support: partial
     bypass_host_loop:
         support: none
