@@ -3,8 +3,7 @@
 # Copyright: (c) 2012, Matt Wright <matt@nobien.net>
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-from __future__ import absolute_import, division, print_function
-__metaclass__ = type
+from __future__ import annotations
 
 
 DOCUMENTATION = '''
@@ -12,8 +11,8 @@ DOCUMENTATION = '''
 module: pip
 short_description: Manages Python library dependencies
 description:
-     - "Manage Python library dependencies. To use this module, one of the following keys is required: C(name)
-       or C(requirements)."
+     - "Manage Python library dependencies. To use this module, one of the following keys is required: O(name)
+       or O(requirements)."
 version_added: "0.7"
 options:
   name:
@@ -24,7 +23,7 @@ options:
     elements: str
   version:
     description:
-      - The version number to install of the Python library specified in the I(name) parameter.
+      - The version number to install of the Python library specified in the O(name) parameter.
     type: str
   requirements:
     description:
@@ -53,17 +52,17 @@ options:
   virtualenv_command:
     description:
       - The command or a pathname to the command to create the virtual
-        environment with. For example C(pyvenv), C(virtualenv),
-        C(virtualenv2), C(~/bin/virtualenv), C(/usr/local/bin/virtualenv).
+        environment with. For example V(pyvenv), V(virtualenv),
+        V(virtualenv2), V(~/bin/virtualenv), V(/usr/local/bin/virtualenv).
     type: path
     default: virtualenv
     version_added: "1.1"
   virtualenv_python:
     description:
       - The Python executable used for creating the virtual environment.
-        For example C(python3.5), C(python2.7). When not specified, the
+        For example V(python3.12), V(python2.7). When not specified, the
         Python version used to run the ansible module is used. This parameter
-        should not be used when C(virtualenv_command) is using C(pyvenv) or
+        should not be used when O(virtualenv_command) is using V(pyvenv) or
         the C(-m venv) module.
     type: str
     version_added: "2.0"
@@ -94,9 +93,9 @@ options:
     description:
       - The explicit executable or pathname for the pip executable,
         if different from the Ansible Python interpreter. For
-        example C(pip3.3), if there are both Python 2.7 and 3.3 installations
+        example V(pip3.3), if there are both Python 2.7 and 3.3 installations
         in the system and you want to run pip for the Python 3.3 installation.
-      - Mutually exclusive with I(virtualenv) (added in 2.1).
+      - Mutually exclusive with O(virtualenv) (added in 2.1).
       - Does not affect the Ansible Python interpreter.
       - The setuptools package must be installed for both the Ansible Python interpreter
         and for the version of Python specified by this option.
@@ -111,6 +110,13 @@ options:
         to specify desired umask mode as an octal string, (e.g., "0022").
     type: str
     version_added: "2.1"
+  break_system_packages:
+    description:
+      - Allow pip to modify an externally-managed Python installation as defined by PEP 668.
+      - This is typically required when installing packages outside a virtual environment on modern systems.
+    type: bool
+    default: false
+    version_added: "2.17"
 extends_documentation_fragment:
   -  action_common_attributes
 attributes:
@@ -121,20 +127,22 @@ attributes:
     platform:
         platforms: posix
 notes:
+   - Python installations marked externally-managed (as defined by PEP668) cannot be updated by pip versions >= 23.0.1 without the use of
+     a virtual environment or setting the O(break_system_packages) option.
    - The virtualenv (U(http://www.virtualenv.org/)) must be
      installed on the remote host if the virtualenv parameter is specified and
      the virtualenv needs to be created.
    - Although it executes using the Ansible Python interpreter, the pip module shells out to
-     run the actual pip command, so it can use any pip version you specify with I(executable).
+     run the actual pip command, so it can use any pip version you specify with O(executable).
      By default, it uses the pip version for the Ansible Python interpreter. For example, pip3 on python 3, and pip2 or pip on python 2.
    - The interpreter used by Ansible
      (see R(ansible_python_interpreter, ansible_python_interpreter))
      requires the setuptools package, regardless of the version of pip set with
-     the I(executable) option.
+     the O(executable) option.
 requirements:
 - pip
 - virtualenv
-- setuptools
+- setuptools or packaging
 author:
 - Matt Wright (@mattupstate)
 '''
@@ -234,6 +242,26 @@ EXAMPLES = '''
     name: bottle
     umask: "0022"
   become: True
+
+- name: Run a module inside a virtual environment
+  block:
+    - name: Ensure the virtual environment exists
+      pip:
+        name: psutil
+        virtualenv: "{{ venv_dir }}"
+        # On Debian-based systems the correct python*-venv package must be installed to use the `venv` module.
+        virtualenv_command: "{{ ansible_python_interpreter }} -m venv"
+
+    - name: Run a module inside the virtual environment
+      wait_for:
+        port: 22
+      vars:
+        # Alternatively, use a block to affect multiple tasks, or use set_fact to affect the remainder of the playbook.
+        ansible_python_interpreter: "{{ venv_python }}"
+
+  vars:
+    venv_dir: /tmp/pick-a-better-venv-path
+    venv_python: "{{ venv_dir }}/bin/python"
 '''
 
 RETURN = '''
@@ -264,6 +292,7 @@ virtualenv:
   sample: "/tmp/virtualenv"
 '''
 
+import argparse
 import os
 import re
 import sys
@@ -271,30 +300,45 @@ import tempfile
 import operator
 import shlex
 import traceback
-import types
 
 from ansible.module_utils.compat.version import LooseVersion
 
-SETUPTOOLS_IMP_ERR = None
+PACKAGING_IMP_ERR = None
+HAS_PACKAGING = False
+HAS_SETUPTOOLS = False
 try:
-    from pkg_resources import Requirement
+    from packaging.requirements import Requirement as parse_requirement
+    HAS_PACKAGING = True
+except Exception:
+    # This is catching a generic Exception, due to packaging on EL7 raising a TypeError on import
+    HAS_PACKAGING = False
+    PACKAGING_IMP_ERR = traceback.format_exc()
+    try:
+        from pkg_resources import Requirement
+        parse_requirement = Requirement.parse  # type: ignore[misc,assignment]
+        del Requirement
+        HAS_SETUPTOOLS = True
+    except ImportError:
+        pass
 
-    HAS_SETUPTOOLS = True
-except ImportError:
-    HAS_SETUPTOOLS = False
-    SETUPTOOLS_IMP_ERR = traceback.format_exc()
-
-from ansible.module_utils._text import to_native
+from ansible.module_utils.common.text.converters import to_native
 from ansible.module_utils.basic import AnsibleModule, is_executable, missing_required_lib
 from ansible.module_utils.common.locale import get_best_parsable_locale
-from ansible.module_utils.six import PY3
 
 
 #: Python one-liners to be run at the command line that will determine the
 # installed version for these special libraries.  These are libraries that
 # don't end up in the output of pip freeze.
-_SPECIAL_PACKAGE_CHECKERS = {'setuptools': 'import setuptools; print(setuptools.__version__)',
-                             'pip': 'import pkg_resources; print(pkg_resources.get_distribution("pip").version)'}
+_SPECIAL_PACKAGE_CHECKERS = {
+    'importlib': {
+        'setuptools': 'from importlib.metadata import version; print(version("setuptools"))',
+        'pip': 'from importlib.metadata import version; print(version("pip"))',
+    },
+    'pkg_resources': {
+        'setuptools': 'import setuptools; print(setuptools.__version__)',
+        'pip': 'import pkg_resources; print(pkg_resources.get_distribution("pip").version)',
+    }
+}
 
 _VCS_RE = re.compile(r'(svn|git|hg|bzr)\+')
 
@@ -307,6 +351,18 @@ def _is_vcs_url(name):
     return re.match(_VCS_RE, name)
 
 
+def _is_venv_command(command):
+    venv_parser = argparse.ArgumentParser()
+    venv_parser.add_argument('-m', type=str)
+    argv = shlex.split(command)
+    if argv[0] == 'pyvenv':
+        return True
+    args, dummy = venv_parser.parse_known_args(argv[1:])
+    if args.m == 'venv':
+        return True
+    return False
+
+
 def _is_package_name(name):
     """Test whether the name is a package name or a version specifier."""
     return not name.lstrip().startswith(tuple(op_dict.keys()))
@@ -394,15 +450,7 @@ def _is_present(module, req, installed_pkgs, pkg_command):
 
 
 def _get_pip(module, env=None, executable=None):
-    # Older pip only installed under the "/usr/bin/pip" name.  Many Linux
-    # distros install it there.
-    # By default, we try to use pip required for the current python
-    # interpreter, so people can use pip to install modules dependencies
-    candidate_pip_basenames = ('pip2', 'pip')
-    if PY3:
-        # pip under python3 installs the "/usr/bin/pip3" name
-        candidate_pip_basenames = ('pip3',)
-
+    candidate_pip_basenames = ('pip3',)
     pip = None
     if executable is not None:
         if os.path.isabs(executable):
@@ -459,7 +507,7 @@ def _have_pip_module():  # type: () -> bool
     except ImportError:
         find_spec = None  # type: ignore[assignment] # type: ignore[no-redef]
 
-    if find_spec:
+    if find_spec:  # type: ignore[truthy-function]
         # noinspection PyBroadException
         try:
             # noinspection PyUnresolvedReferences
@@ -491,7 +539,7 @@ def _fail(module, cmd, out, err):
     module.fail_json(cmd=cmd, msg=msg)
 
 
-def _get_package_info(module, package, env=None):
+def _get_package_info(module, package, python_bin=None):
     """This is only needed for special packages which do not show up in pip freeze
 
     pip and setuptools fall into this category.
@@ -499,20 +547,19 @@ def _get_package_info(module, package, env=None):
     :returns: a string containing the version number if the package is
         installed.  None if the package is not installed.
     """
-    if env:
-        opt_dirs = ['%s/bin' % env]
-    else:
-        opt_dirs = []
-    python_bin = module.get_bin_path('python', False, opt_dirs)
-
     if python_bin is None:
+        return
+
+    discovery_mechanism = 'pkg_resources'
+    importlib_rc = module.run_command([python_bin, '-c', 'import importlib.metadata'])[0]
+    if importlib_rc == 0:
+        discovery_mechanism = 'importlib'
+
+    rc, out, err = module.run_command([python_bin, '-c', _SPECIAL_PACKAGE_CHECKERS[discovery_mechanism][package]])
+    if rc:
         formatted_dep = None
     else:
-        rc, out, err = module.run_command([python_bin, '-c', _SPECIAL_PACKAGE_CHECKERS[package]])
-        if rc:
-            formatted_dep = None
-        else:
-            formatted_dep = '%s==%s' % (package, out.strip())
+        formatted_dep = '%s==%s' % (package, out.strip())
     return formatted_dep
 
 
@@ -541,16 +588,13 @@ def setup_virtualenv(module, env, chdir, out, err):
     virtualenv_python = module.params['virtualenv_python']
     # -p is a virtualenv option, not compatible with pyenv or venv
     # this conditional validates if the command being used is not any of them
-    if not any(ex in module.params['virtualenv_command'] for ex in ('pyvenv', '-m venv')):
+    if not _is_venv_command(module.params['virtualenv_command']):
         if virtualenv_python:
             cmd.append('-p%s' % virtualenv_python)
-        elif PY3:
-            # Ubuntu currently has a patch making virtualenv always
-            # try to use python2.  Since Ubuntu16 works without
-            # python2 installed, this is a problem.  This code mimics
-            # the upstream behaviour of using the python which invoked
-            # virtualenv to determine which python is used inside of
-            # the virtualenv (when none are specified).
+        else:
+            # This code mimics the upstream behaviour of using the python
+            # which invoked virtualenv to determine which python is used
+            # inside of the virtualenv (when none are specified).
             cmd.append('-p%s' % sys.executable)
 
     # if venv or pyvenv are used and virtualenv_python is defined, then
@@ -590,13 +634,15 @@ class Package:
             separator = '==' if version_string[0].isdigit() else ' '
             name_string = separator.join((name_string, version_string))
         try:
-            self._requirement = Requirement.parse(name_string)
+            self._requirement = parse_requirement(name_string)
             # old pkg_resource will replace 'setuptools' with 'distribute' when it's already installed
-            if self._requirement.project_name == "distribute" and "setuptools" in name_string:
+            project_name = Package.canonicalize_name(
+                getattr(self._requirement, 'name', None) or getattr(self._requirement, 'project_name', None)
+            )
+            if project_name == "distribute" and "setuptools" in name_string:
                 self.package_name = "setuptools"
-                self._requirement.project_name = "setuptools"
             else:
-                self.package_name = Package.canonicalize_name(self._requirement.project_name)
+                self.package_name = project_name
             self._plain_package = True
         except ValueError as e:
             pass
@@ -604,7 +650,7 @@ class Package:
     @property
     def has_version_specifier(self):
         if self._plain_package:
-            return bool(self._requirement.specs)
+            return bool(getattr(self._requirement, 'specifier', None) or getattr(self._requirement, 'specs', None))
         return False
 
     def is_satisfied_by(self, version_to_test):
@@ -641,7 +687,7 @@ def main():
 
     module = AnsibleModule(
         argument_spec=dict(
-            state=dict(type='str', default='present', choices=state_map.keys()),
+            state=dict(type='str', default='present', choices=list(state_map.keys())),
             name=dict(type='list', elements='str'),
             version=dict(type='str'),
             requirements=dict(type='str'),
@@ -654,15 +700,16 @@ def main():
             chdir=dict(type='path'),
             executable=dict(type='path'),
             umask=dict(type='str'),
+            break_system_packages=dict(type='bool', default=False),
         ),
         required_one_of=[['name', 'requirements']],
         mutually_exclusive=[['name', 'requirements'], ['executable', 'virtualenv']],
         supports_check_mode=True,
     )
 
-    if not HAS_SETUPTOOLS:
-        module.fail_json(msg=missing_required_lib("setuptools"),
-                         exception=SETUPTOOLS_IMP_ERR)
+    if not HAS_SETUPTOOLS and not HAS_PACKAGING:
+        module.fail_json(msg=missing_required_lib("packaging"),
+                         exception=PACKAGING_IMP_ERR)
 
     state = module.params['state']
     name = module.params['name']
@@ -702,6 +749,9 @@ def main():
             if not os.path.exists(os.path.join(env, 'bin', 'activate')):
                 venv_created = True
                 out, err = setup_virtualenv(module, env, chdir, out, err)
+            py_bin = os.path.join(env, 'bin', 'python')
+        else:
+            py_bin = module.params['executable'] or sys.executable
 
         pip = _get_pip(module, env, module.params['executable'])
 
@@ -755,6 +805,11 @@ def main():
         if extra_args:
             cmd.extend(shlex.split(extra_args))
 
+        if module.params['break_system_packages']:
+            # Using an env var instead of the `--break-system-packages` option, to avoid failing under pip 23.0.0 and earlier.
+            # See: https://github.com/pypa/pip/pull/11780
+            os.environ['PIP_BREAK_SYSTEM_PACKAGES'] = '1'
+
         if name:
             cmd.extend(to_native(p) for p in packages)
         elif requirements:
@@ -784,7 +839,7 @@ def main():
                     # So we need to get those via a specialcase
                     for pkg in ('setuptools', 'pip'):
                         if pkg in name:
-                            formatted_dep = _get_package_info(module, pkg, env)
+                            formatted_dep = _get_package_info(module, pkg, py_bin)
                             if formatted_dep is not None:
                                 pkg_list.append(formatted_dep)
                                 out += '%s\n' % formatted_dep
@@ -798,7 +853,7 @@ def main():
 
         out_freeze_before = None
         if requirements or has_vcs:
-            _, out_freeze_before, _ = _get_packages(module, pip, chdir)
+            dummy, out_freeze_before, dummy = _get_packages(module, pip, chdir)
 
         rc, out_pip, err_pip = module.run_command(cmd, path_prefix=path_prefix, cwd=chdir)
         out += out_pip
@@ -815,7 +870,7 @@ def main():
             if out_freeze_before is None:
                 changed = 'Successfully installed' in out_pip
             else:
-                _, out_freeze_after, _ = _get_packages(module, pip, chdir)
+                dummy, out_freeze_after, dummy = _get_packages(module, pip, chdir)
                 changed = out_freeze_before != out_freeze_after
 
         changed = changed or venv_created
