@@ -31,11 +31,6 @@ from termios import TIOCGWINSZ
 # CAUTION: Avoid third-party imports in this module whenever possible.
 #          Any third-party imports occurring here will result in an error if they are vendored by ansible-core.
 
-try:
-    from typing_extensions import TypeGuard  # TypeGuard was added in Python 3.10
-except ImportError:
-    TypeGuard = None
-
 from .locale_util import (
     LOCALE_WARNING,
     CONFIGURED_LOCALE,
@@ -1003,7 +998,7 @@ def retry(func: t.Callable[..., TValue], ex_type: t.Type[BaseException] = Subpro
 def parse_to_list_of_dict(pattern: str, value: str) -> list[dict[str, str]]:
     """Parse lines from the given value using the specified pattern and return the extracted list of key/value pair dictionaries."""
     matched = []
-    unmatched = []
+    unmatched: list[str] = []
 
     for line in value.splitlines():
         match = re.search(pattern, line)
@@ -1155,7 +1150,7 @@ def verify_sys_executable(path: str) -> t.Optional[str]:
     return expected_executable
 
 
-def type_guard(sequence: c.Sequence[t.Any], guard_type: t.Type[C]) -> TypeGuard[c.Sequence[C]]:
+def type_guard(sequence: c.Sequence[t.Any], guard_type: t.Type[C]) -> t.TypeGuard[c.Sequence[C]]:
     """
     Raises an exception if any item in the given sequence does not match the specified guard type.
     Use with assert so that type checkers are aware of the type guard.
